// capture.cpp
#include "VidCapture.h"
#include <stdio.h>
#include <string>
#include <vector>
#include <algorithm>
using namespace std;

static CVVidCapture* vidCap = 0;
static CRITICAL_SECTION pbuffer_crit;
static int w, h;
static unsigned int *clean_buffer = 0;
static unsigned int *scratch_buffer = 0;
bool new_data = false;

static bool capCallback (CVRES status, CVImage* image, void* userParam)
{
	// copy pixels to the scratch buffer
	unsigned char *pixels = image->GetRawDataPtr();
	unsigned char *dst = (unsigned char*) scratch_buffer;
	for (int i=0; i<w*h; i++)
	{
		*dst++ = *pixels++; // R
		*dst++ = *pixels++; // G
		*dst++ = *pixels++; // B
		*dst++ = 0; // A
	}

	// swap clean and scratch buffers
	// make sure that no other thread is using them
	EnterCriticalSection(&pbuffer_crit);
	std::swap(clean_buffer, scratch_buffer);
	new_data = true;
	LeaveCriticalSection(&pbuffer_crit);

	return true;
}

class CameraMode
{
public:
	int width, height;
	int fps;
	int id;

	CameraMode(int width, int height, int fps, int id)
	{
		this->width = width;
		this->height = height;
		this->fps = fps;
		this->id = id;
	}

	bool operator < (const CameraMode &m) const
	{
		if (fps < m.fps) return true;
		if (width * height < m.width * m.height) return true;
		if (width < m.width) return true;
		return false;
	}
};

bool StartCam()
{
	vidCap = CVPlatform::GetPlatform()->AcquireVideoCapture();
	if (CVSUCCESS(vidCap->Init()))
	{
		printf("I: Initialized video capture object.\n");
	}
	else
	{
		printf("E: Error initializing video capture object.\n");
		CVPlatform::GetPlatform()->Release(vidCap);                  
		return false;
	}

	int numDevices = 0;
    if (CVFAILED(vidCap->GetNumDevices(numDevices)))
    {
		printf("E: Failed enumerating devices.\n");
		vidCap->Uninit();
		CVPlatform::GetPlatform()->Release(vidCap);      
		return false;
    }

	if (!numDevices)
	{
		printf("E: No webcams found.\n");
		vidCap->Uninit();
		CVPlatform::GetPlatform()->Release(vidCap); 
		return false;
	}

	// initialize first webcam
	CVVidCapture::VIDCAP_DEVICE devInfo;
	if (!CVSUCCESS(vidCap->GetDeviceInfo(0, devInfo)))
	{
		printf("E: failed to initialize webcam\n");
		vidCap->Uninit();
		CVPlatform::GetPlatform()->Release(vidCap); 
		return false;
	}

	printf("I: Initializing device %d: %s\n", 0, devInfo.DeviceString);

	// Now connect to the selected device.
	if (!CVSUCCESS(vidCap->Connect(0)))
	{
		printf("E: Connection failed.\n");
		vidCap->Uninit();
		CVPlatform::GetPlatform()->Release(vidCap);      
		return false;
	}
	printf("I: Connection succeeded.\n");

	CVVidCapture::VIDCAP_MODE modeInfo;
	int numModes = 0;
	vidCap->GetNumSupportedModes(numModes);

	// Dump each mode 
	std::vector<CameraMode> modes;
	for (int curmode = 0; curmode < numModes; curmode++)
	{
		if (CVSUCCESS(vidCap->GetModeInfo(curmode, modeInfo)))
		{
			printf(  "I: Available mode: %d, %dx%d @ %d frames/sec (%s)\n",
					curmode, 
					modeInfo.XRes, 
					modeInfo.YRes,
					modeInfo.EstFrameRate,
					vidCap->GetFormatModeName(modeInfo.InputFormat));

			modes.push_back(CameraMode(modeInfo.XRes, 
				modeInfo.YRes, modeInfo.EstFrameRate, curmode));
		} 
	}

	// choose best mode
	std::sort(modes.begin(), modes.end());

	if (!modes.size())
	{
		printf("E: No video modes are available.\n");
		vidCap->Uninit();
		CVPlatform::GetPlatform()->Release(vidCap);  
		return false;
	}
 
	int bestmode = (int) modes[modes.size()-1].id;
	printf("I: Selected webcam mode: %d.\n", bestmode);

    if (CVFAILED(vidCap->SetMode(bestmode)))
    {
		printf("Error setting video mode.\n");
		vidCap->Uninit();
		CVPlatform::GetPlatform()->Release(vidCap);  
		return false;
    }

	// Get the mode info of our selected mode and print it.
	if (CVFAILED(vidCap->GetCurrentMode(modeInfo)))
	{
		printf("E: Error activating mode!\n");
		vidCap->Uninit();
		CVPlatform::GetPlatform()->Release(vidCap);  
		return false;
	}
	
	printf("I: Activated mode: %dx%d\n", modeInfo.XRes, modeInfo.YRes);

	// initialize pixel buffers
	w = modeInfo.XRes;
	h = modeInfo.YRes;
	if (clean_buffer) free(clean_buffer);
	clean_buffer = (unsigned int*) malloc(w*h*sizeof(unsigned int));
	if (scratch_buffer) free(scratch_buffer);
	scratch_buffer = (unsigned int*) malloc(w*h*sizeof(unsigned int));

    // Set contrast to 50% if supported. Only some video capture devices
    // will support the properties.
    long minval,maxval;
    if (CVSUCCESS(vidCap->GetPropertyInfo( CVVidCapture::CAMERAPROP_CONTRAST,
                                           0,
                                           0,
                                           &minval, 
                                           &maxval)))
    {
		printf("I: Contrast supported\n", minval, maxval);    
       vidCap->SetProperty( CVVidCapture::CAMERAPROP_CONTRAST,
                            (minval+maxval)/2);
    }
    else
    {
		printf("I: Contrast unsupported.\n");
    }
 
    // Now set the brightness if available.
    if (CVSUCCESS(vidCap->GetPropertyInfo( CVVidCapture::CAMERAPROP_BRIGHT,
                                           0,
                                           0,
                                           &minval, 
                                           &maxval)))
    {
		printf("I: Brightness supported\n", minval, maxval);     
       vidCap->SetProperty( CVVidCapture::CAMERAPROP_BRIGHT,
                            (minval+maxval)/2);
    }
    else
    {
		printf("I: Brightness unsupported.\n");
    }

	// create pbuffer critical section
	InitializeCriticalSection(&pbuffer_crit);

	// start capturing
	printf("I: Starting color image capture...\n");
	bool started = CVSUCCESS(vidCap->StartImageCap(CVImage::CVIMAGE_RGB24, capCallback, 0));
	if (!started)
	{
		printf("E: Error starting capture....\n");
		vidCap->Uninit();
		CVPlatform::GetPlatform()->Release(vidCap);  
		return false;
	}      

	return true;
}

int GetCamWidth()
{
	return w;
}

int GetCamHeight()
{
	return h;
}

bool NewCamData()
{
	return new_data;
}

unsigned int *BeginCamData() {
	EnterCriticalSection(&pbuffer_crit);
	return clean_buffer;
}

void EndCamData() {
	new_data = false;
	LeaveCriticalSection(&pbuffer_crit);
}

void StopCam()
{
	// Stop the capture
	printf("I: Stopping image capture...\n");
	vidCap->Stop();

	// delete the critical section
	DeleteCriticalSection(&pbuffer_crit);

	// free intermediate buffers
	if (clean_buffer)
	{
		free(clean_buffer);
		clean_buffer = 0;
	}
	if (scratch_buffer)
	{
		free(scratch_buffer);
		scratch_buffer = 0;
	}

	 // Always disconnect from the device when you're done
	printf("I: Disconnecting...\n");
    vidCap->Disconnect();   
 
    // And uninitialize before deleting it
	printf("I: Uninitializing video capture...\n");
    vidCap->Uninit();
 
    // Release video capture object.
    CVPlatform::GetPlatform()->Release(vidCap); 
}