// cluster.cpp

#include "cluster.h"

#define MIN_CLUSTER_SIZE 1

struct v2i
{
	int x,  y;

	v2i(int x, int y) {this->x = x; this->y = y;}
};

inline bool PixelMatch(int r1, int g1, int b1, 
					   int r2, int g2, int b2,
					   int threshold)
{
	// calculate square distance;
	int dr = r1 - r2;
	int dg = g1 - g2;
	int db = b1 - b2;

	if (dr*dr + dg*dg + db*db <= threshold) return true;
	return false;
}

inline bool ClearRow(int r, int g, int b, int w,
					 unsigned char *row, int threshold)
{
	unsigned char *pixel = row;
	for (int i=0; i<w; i++)
	{
		int r2 = *pixel++;
		int g2 = *pixel++;
		int b2 = *pixel++;
		pixel++;
		if (PixelMatch(r, g, b, r2, g2, b2, threshold))
			return false;
	}

	return true;
}

inline bool ClearColumn(int r, int g, int b, int w, int h,
					 unsigned char *column, int threshold)
{
	unsigned char *pixel = column;
	for (int i=0; i<h; i++)
	{
		int r2 = pixel[0];
		int g2 = pixel[1];
		int b2 = pixel[2];
		pixel += 4 * w;
		if (PixelMatch(r, g, b, r2, g2, b2, threshold))
			return false;
	}

	return true;
}

std::vector<v2i> Columns(int r, int g, int b, int w, int h,
						 unsigned char *image, int threshold)
{
	std::vector<v2i> ret;
	bool started = false;
	int last_start_pos = 0;
	unsigned char *col = image;
	for (int i=0; i<w; i++)
	{
		bool col_clear = ClearColumn(r, g, b, w, h, col, threshold);
		if (col_clear)
		{
			if (started)
			{
				if (i-last_start_pos > MIN_CLUSTER_SIZE)
					ret.push_back(v2i(last_start_pos, i));
			}
			started = false;
		}
		else
		{
			if (!started)
			{
				last_start_pos = i;
			}
			started = true;
		}

		col += 4;
	}

	if (started)
	{
		if (w-last_start_pos > MIN_CLUSTER_SIZE)
			ret.push_back(v2i(last_start_pos, w));
	}

	return ret;
}

std::vector<v2i> Rows(int r, int g, int b, int w, int h,
						 unsigned char *image, int threshold)
{
	std::vector<v2i> ret;
	bool started = false;
	int last_start_pos = 0;
	unsigned char *row = image;
	for (int i=0; i<h; i++)
	{
		bool row_clear = ClearRow(r, g, b, w, row, threshold);
		if (row_clear)
		{
			if (started)
			{
				if (i-last_start_pos > MIN_CLUSTER_SIZE) 
					ret.push_back(v2i(last_start_pos, i));
			}
			started = false;
		}
		else
		{
			if (!started)
			{
				last_start_pos = i;
			}
			started = true;
		}

		row += 4 * w;
	}

	if (started)
	{
		if (h-last_start_pos > MIN_CLUSTER_SIZE)
		ret.push_back(v2i(last_start_pos, h));
	}

	return ret;
}

void ProcessImage(int w, int h, unsigned char *pixels)
{
	unsigned char *row = pixels;
	for (int j=0; j<h; j++)
	{
		/*
		for (int i=0; i<w; i++)
		{
			int r = row[0];
			int g = row[1];
			int b = row[2];
			
			int dr = (255 - r);
			int dg = g;
			int db = b;
			int dist = dr*dr + dg*dg + db*db;
			dist /= 255;
			if (dist > 255) dist = 255;

			if (dist < 50)
			{
				r = b = 255;
				g = 0;
			}
			
			row[0] = r;
			row[1] = g;
			row[2] = b;

			row += 4;
		}*/
		if (ClearRow(255, 0, 0, w, row, 50 * 255))
		{
			memset(row, 0, 4 * w);
		}
		row += 4 * w;
	}
}

static inline bool ProcessCluster(int r, int g, int b, int w, int h,
			   unsigned char *image, int threshold, 
			   v2i row, v2i col, Cluster *cl)
{
	int samples = 0;
	float cx = 0;
	float cy = 0;
	for (int j=row.x; j<row.y; j++)
	{
		for (int i=col.x; i<col.y; i++)
		{
			unsigned char *pixel = image + 4 * (i + w * j);
			int ir = pixel[0];
			int ig = pixel[1];
			int ib = pixel[2];

			if (PixelMatch(r, g, b, ir, ig, ib, threshold))
			{
				samples ++;
				cx += i;
				cy += j;
			}
		}
	}

	if (!samples) return false;

	cx /= samples;
	cy /= samples;
	cl->cx = cx;
	cl->cy = cy;
	cl->sample_count = samples;

	return true;
}

std::vector<Cluster> ClusterImage(int w, int h, unsigned char *pixels,
								  int r, int g, int b, int threshold)
{
	std::vector<v2i> columns = Columns(r, g, b, w, h, pixels, threshold);
	std::vector<v2i> rows = Rows(r, g, b, w, h, pixels, threshold);


	printf("%d rows, %d columns\n", rows.size(), columns.size());

	std::vector<Cluster> ret;

	for (int i=0; i<(int)columns.size(); i++)
	{
		for (int j=0; j<(int)rows.size(); j++)
		{
			v2i col = columns[i];
			v2i row = rows[j];

			Cluster cl;
			cl.x = (float) col.x;
			cl.y = (float) row.x;
			cl.w = (float) col.y - col.x;
			cl.h = (float) row.y - row.x;

			if (ProcessCluster(r, g, b, w, h, pixels, threshold,
				row, col, &cl))
			{
				ret.push_back(cl);
			}
		}
	}

	return ret;
}
